/* 
 * File:   aqReceiver.h
 * Author: matt
 *
 * Created on July 5, 2011, 12:49 AM
 */

#ifndef AQRECEIVER_H
#define	AQRECEIVER_H

#include <qpid/messaging/Message.h>
#include <qpid/messaging/Receiver.h>
#include <qpid/messaging/Session.h>
#include "robokind/messaging/qpidavroUtils.h"

using namespace std;
using namespace qpid::messaging;

namespace robokind{
    class AQReceiver {
        private:
            Session* mySession;
            Receiver myReceiver;
            Message myMessage;
            bool myAutoAcknowledge;
        public:
            AQReceiver(Session *session, const Receiver &receiver, bool autoAcknowledge=true)
                    :mySession(session), myReceiver(receiver), myAutoAcknowledge(autoAcknowledge){}

            AQReceiver(Session* session, const char* destination, bool autoAcknowledge=true)
                    :mySession(session), myAutoAcknowledge(autoAcknowledge){
                myReceiver = session->createReceiver(destination);
            }

            bool isAutoAcknowledge(){
                return myAutoAcknowledge;
            }

            void setAutoAcknowledge(bool val){
                myAutoAcknowledge = val;
            }

            template<typename T> T* fetch(Duration timeout=Duration::FOREVER){
                myReceiver.fetch(myMessage, timeout);
                T* t = unpackMessage<T>(myMessage);
                if(t == NULL){
                    return NULL;
                }
                if(myAutoAcknowledge){
                    mySession->acknowledge(myMessage);
                }
                return t;
            }
            
            Message fetchRaw(Duration timeout=Duration::FOREVER){
                return myReceiver.fetch(timeout);
            }
            
            Message* getLastMessage(){
                return &myMessage;
            }

            void acknowledge(bool sync=false){
                mySession->acknowledge(sync);
            }
    };
}
#endif	/* AQRECEIVER_H */

