/* 
 * File:   AQServiceProvider.h
 * Author: matt
 *
 * Created on July 14, 2011, 5:24 PM
 */

#ifndef AQSERVICEPROVIDER_H
#define	AQSERVICEPROVIDER_H

#include <string.h>
#include <iostream>
#include <exception>

#include "robokind/common/logger.h"
#include "robokind/messaging/AQService.h"
#include "robokind/messaging/AQStream.h"
#include "robokind/messaging/protocol/AQServiceCommand.h"
#include "robokind/messaging/protocol/AQServiceError.h"

using namespace qpid::messaging;
using namespace std;

namespace robokind{
    template<class T>
    class AQServiceProvider {
        private:
            AQReceiver* myCommandReceiver;
            AQSender* myErrorSender;
            AQService<T>* myService;
            
            void commandLoop(){
                bool run = true;
                while(run){
                    try{
                        Message message = myCommandReceiver->fetchRaw();
                        string contentType = message.getContentType();
                        cout << "Service Provider received message with content type: " << contentType << endl;
                        if(contentType == "application/service-command"){
                            run = handleCommand(message);
                        }else{
                            handleOther(message);
                        }
                    }catch(const NoMessageAvailable &ex){}
                }
            }
            
            bool handleCommand(Message &message){
                AQServiceCommand* cmd = unpackMessage<AQServiceCommand>(message);
                cout << "Command Received: " << cmd->command << endl;
                myCommandReceiver->acknowledge();
                switch(cmd->command){
                    case START: myService->start(); break;
                    case PAUSE: myService->pause(); break;
                    case RESUME: myService->resume(); break;
					case STOP: myService->stop(); return false;
                }
                return true;
            }
            
            void handleOther(Message &message){
                cout << "Handling Custon Service Command" << endl;
                myService->handleCommandMessage(message);
                myCommandReceiver->acknowledge();
            }
            
            T* fetchConfig(){
                try{
                    T* t = myCommandReceiver->fetch<T>();
                    if(t == NULL){
                        Message* msg = myCommandReceiver->getLastMessage();
                        if(msg == NULL){
                            return NULL;
                        }
                        cout << "Message is not the required config.  Discarding message.  Type: " 
                                << msg->getContentType() <<endl;
                        myCommandReceiver->acknowledge();
                    }
                    return t;
                }catch(const MessagingException &ex){
                    cout << "Messaging Error fetching config: " << ex.what() << endl;
                }catch(const Exception &ex){
                    cout << "Error fetching config: " << ex.what() << endl;
                }catch(...){
                    cout << "Unknown Error fetching config" << endl;
                }
                return NULL;
            }

			void initialize(){
				if(myService->isInitialized()){
					return;
				}
                T* t = NULL;
                while(t == NULL){
                    t = fetchConfig();
                }
                myService->initialize(t);
			}
            
        public:
            AQServiceProvider(AQService<T>* service, AQReceiver* cmdRec, AQSender* errSnd)
                : myService(service), myCommandReceiver(cmdRec), myErrorSender(errSnd){}
                
            bool start(){
				initialize();
                commandLoop();
				return true;
            }
    };
}

#endif	/* AQSERVICEPROVIDER_H */

