/* 
 * File:   main.cpp
 * Author: matt
 *
 * Created on November 12, 2011, 4:29 PM
 */

#include <cstdlib>
#include <stdio.h>
#include "robokind/messaging/AQStream.h"
#include "robokind/messaging/AQSender.h"
#include "robokind/messaging/AQService.h"
#include "robokind/messaging/AQServiceProvider.h"
#include "robokind/vision/display/ImageDisplayService.h"
#include "robokind/vision/display/protocol/ImageDisplayConfig.h"
#include "robokind/vision/camera/protocol/CameraConfig.h"
#include "robokind/vision/proc/facedetect/protocol/FaceDetectConfig.h"

using namespace std;
using namespace qpid::messaging;
using namespace robokind;

const char* connectionStr = "localhost:5672";
const char* connectionOpt = "{username:admin,password:admin}";
const char* destVidQ = "vision.VideoTopic1; {create: always, node: {type: topic}}";
const char* destFD = "vision.FaceTopic1; {create: always, node: {type: topic}}";
const char* destDspCmd = "vision.DspCmdQueue1; {create: always, node: {type: queue}}";
const char* destDspErr = "vision.DspErrQueue1; {create: always, node: {type: queue}}";
const char* destCamCmd = "vision.CamCmdQueue1; {create: always, node: {type: queue}}";
const char* destFDCmd = "vision.FaceCmdQueue1; {create: always, node: {type: queue}}";

ImageDisplayConfig* getDisplayConfig(){
    ImageDisplayConfig* conf = new ImageDisplayConfig();
    conf->brokerAddress = connectionStr;
    conf->connectionOptions = connectionOpt;
    conf->imageQueue = destVidQ;
    conf->annotationQueue = destFD;
    conf->drawRate = 10;
    conf->windowName = "Display";
    return conf;
}

void displayService(){
    AQService<ImageDisplayConfig>* ids = new ImageDisplayService();
    ids->initialize(getDisplayConfig());
    ids->start();
}

void sendCmd(AQSender* sender, ServiceCommand sc){
    AQServiceCommand cmd;
    cmd.command = sc;
    sender->send<AQServiceCommand>(&cmd, "application/service-command", true);
}

void startProcs(const char* camDest, const char* fdDest){
    AQStream* myStream;
    myStream = new AQStream();
    if(!myStream->connect())
        return;
    AQSender* fdCmdSnd = myStream->createSender(fdDest);
    sendCmd(fdCmdSnd, START);
    AQSender* camCmdSnd = myStream->createSender(camDest);
    sendCmd(camCmdSnd, START);
}

void wait(){
    bool stop = false;
    char k;
    while(!stop){
        k = getchar();
        if(k == 'q'){
            stop = true;
        }
    }
}

int main(int argc, char** argv) {
    cout << "Starting Image Display Service." << endl;
    displayService();
    startProcs(destCamCmd, destFDCmd);
    wait();
    return 0;
}