#include "robokind/vision/camera/CameraService.h"
#include <sys/time.h>

using namespace robokind;

#define DEF_CAMERA_WIDTH 640
#define DEF_CAMERA_HEIGHT 480
#define DEF_CAMERA_CHANNELS 3

bool CameraService::_initialize(CameraConfig* conf){
    AQStream* stream = new AQStream(
            conf->brokerAddress.c_str(), conf->connectionOptions.c_str());
    stream->connect();
    myFrameLength = (int)conf->frameLength;
    myDevice = (int)conf->cameraNumber;
    string dest = conf->destination;
    myVideoSender = stream->createSender(dest.c_str());
    
    myCapture = cvCreateCameraCapture(myDevice);
    int c = conf->imageChannels;
    int w = conf->imageWidth;
    int h = conf->imageHeight;
    
    myConvertColorFlag = (c != DEF_CAMERA_CHANNELS);
    myConvertColorHeader = cvCreateImage(cvSize(DEF_CAMERA_WIDTH,DEF_CAMERA_HEIGHT),IPL_DEPTH_8U,c);
    
    myResizeFlag = (w != DEF_CAMERA_WIDTH || h != DEF_CAMERA_HEIGHT);
    myResizeHeader = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,c);

	return true;
}

void CameraService::close(){
    cvReleaseCapture(&myCapture);
}

void CameraService::threadSafeWorker(){
	int start = cvGetTickCount();
    try{
        IplImage* img = cvQueryFrame(myCapture);
        IplImage* temp = img;
        if(myConvertColorFlag){
            cvConvertImage(temp,myConvertColorHeader);
            temp = myConvertColorHeader;
        }
        if(myResizeFlag){
            cvResize(temp,myResizeHeader);
            temp = myResizeHeader;
        }
        ImageRecord *pimg = packImage(temp);
        myVideoSender->send<ImageRecord>(pimg, "application/image-record", true);
        
        delete pimg;
		
		int end = cvGetTickCount();
        long elapsed = end - start;
        long sleep = myFrameLength - elapsed;
        if(sleep > 0){
            boost::this_thread::sleep(boost::posix_time::milliseconds(sleep));
        }
    }catch(const cv::Exception &ex){
        cout << ex.what() << endl;
    }
}

void CameraService::handleCommandMessage(Message &msg){}


int CameraService::getFrameLength(){
    return myFrameLength;
}

void CameraService::setFrameLength(int len){
    myFrameLength = len;
}